/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.util.*;
import java.sql.*;

/**
  * This program will implement all database related activities
  * for storing records in the warehouse in database as follows:
  *    1) create all tables of the database warehouse
  *    2) delete all tables of the database warehouse
  *    3) get time stamp of the most recent stored access record
  *    4) check if the record is in the database warehouse
  *    5) insert entries into UserDimTbl, ProtocolDimTbl, PathDimTbl, and LogFactTble tables
  *    6) determine sessionID for each web access
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataCollectorModel {
   private Connection con = null;    // represent connection with the database

   /**
     * The constructor will establish connection with the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   ClassNotFoundException         if class specified by driver cannot be located
     *
     * @exception   SQLException                   for sql exception
     */

   public DataCollectorModel () {
      // the singleton object which will store driver and connection names for database access
      Config config = Config.Factory();
      // getting driver
      String driver = config.getDriver();
      // getting connection
      String connection = config.getConnection();

      // the singleton object which will store user id and password for database access
      DBLogin login = DBLogin.Factory();
      // getting user id
      String userID = login.getUserID();
      // getting password
      String password = login.getPassword();

      try {
         // getting reference
         Class.forName ( driver );
         // getting connection
         con = DriverManager.getConnection( connection, userID, password );
      } catch ( ClassNotFoundException cnf ) {
         cnf.printStackTrace();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method closeConnection will disconnect the program from the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   SQLException                   for sql exception
     */

   public void closeConnection() {
      try {
         con.close();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method getLastAccessTime get time stamp of the most recent access record
     * stored in the warehouse
     *
     * @exception   SQLException   for sql exception
     *
     * @return                     time stamp of the most recent web log access record
     */

   public MyDate getLastAccessTime() {
      String timeID = "";
      Statement stmt = null;
      ResultSet result = null;
      String qry;

      MyDate d = null;
      MyDate d1 = null;
      try {
         stmt = con.createStatement();

         // getting time stamp of the most recent web access record
         qry = "SELECT timeID from LogFactTbl";
         result = stmt.executeQuery( qry );
         while ( result != null && result.next() ) {
            timeID = result.getString ( 1 );
            d1 = new MyDate( timeID );

            // comparing time stamps to get time stamp of more recent access
            if( d == null || d1.compareTo( d ) >= 0 ) {
               d = d1;
            }
         }
         result.close();
         stmt.close();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
      return d;
   }

   /**
     * Method isInStore checks if the record is in the warehouse
     *
     * @param       arg                      arg is a array of Strings
     *
     * @exception   SQLException             for sql exception
     *
     * @return                               true if the record is in store
     */

   public boolean isInStore( String[] arg ) {
      boolean flag = false;
      int uID = -1;
      int protID = -1;
      int paID = -1;

      Statement stmt = null;
      ResultSet result = null;
      String qry;

      try {
        stmt = con.createStatement();

         // getting useID from UserDimTble for the record
	 qry = "SELECT userID FROM UserDimTbl WHERE ipAddress = \'" + arg[0] + "\'";
         result = stmt.executeQuery( qry );
         if( result != null && result.next() ) {
            uID = result.getInt ( 1 );
         }
	 result.close();
         // if record has no userID in UserDimTbl,
         // the record is not in the warehouse
	 if( uID == -1 ) {
	    stmt.close();
	    return flag;
	 }

         // getting protocolID from ProtocolDimTble for the record
         qry = "SELECT protocolID FROM ProtocolDimTbl WHERE "
               + "protocolName = \'" + arg[9] + "\' AND "
               + "methodName = \'" + arg[5] + "\' AND " + "status = " + arg[10];
         result = stmt.executeQuery( qry );
         if( result != null && result.next() ) {
            protID = result.getInt ( 1 );
         }
	 result.close();
         // if record has no protocolID in ProtocolDimTbl,
         // the record is not in the warehouse
	 if( protID == -1 ) {
	    stmt.close();
	    return flag;
	 }

         // getting pathID from PathDimTble for the record
	 qry = "SELECT pathID FROM PathDimTbl WHERE pathName = \'"
              + arg[6] + "\' AND " + "fileName = \'"
              + arg[7] + "\' AND " + "fileType = \'"
              + arg[8] + "\'";
         result = stmt.executeQuery( qry );
         if( result != null && result.next() ) {
            paID = result.getInt ( 1 );
         }
	 result.close();
         // if record has no pathID in PathDimTbl,
         // the record is not in the warehouse
	 if( paID == -1 ) {
	    stmt.close();
	    return flag;
	 }

         // logical timeID
         String dateStr = arg[3] + "/" + arg[4];
         String timeID = "";

         int nbyte;
         if( !arg[11].equals( "-" ) ) {
            nbyte = Integer.parseInt( arg[11] );
         } else {
            nbyte = 0;
         }
         // getting the entry in LogFactTbl which match with the access record
	 qry = "SELECT * FROM LogFactTbl WHERE timeID = \'" + dateStr
             + "\' AND " + "userID = " + uID
             + " AND " + "protocolID = " + protID
             + " AND " + "pathID = " + paID
             + " AND " + "nbyte = " + nbyte;
         result = stmt.executeQuery( qry );
         if( result != null && result.next() ) {
            int sID = result.getInt ( 5 );
            // compare time stamps
            if( sID > 0 ) {
               flag = true;
            }
         }
	 result.close();
	 stmt.close();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      } catch(Exception e ){}
      return flag;
   }

   /**
     * Method insertToUserDimTbl will return userID such that
     *    1) if the user information exist in the UserDimTbl,
     *       it will return the existing userID
     *    2) if the user information does not exist in the UserDimTbl,
     *       it will insert the information to the UserDimTbl and
     *       return the new userID
     *
     * @param       ip              ip is IP Address of the user
     *
     * @param       hostName        hostName is host name of the IP Address
     *
     * @param       userName        userName is user name of the IP Address
     *
     * @exception   SQLException    for sql exception
     *
     * @return                      userID for the user
     */

   public int insertToUserDimTbl( String ip,
                                  String hostName,
                                  String userName ) {
      int uID = -1;
      int count = -1;
      boolean flag = false;
      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      try {
         stmt = con.createStatement();
         qry = "SELECT userID FROM UserDimTbl WHERE ipAddress = \'" + ip + "\'";
         result = stmt.executeQuery( qry );

         // if selection to get userID,
         // if the user information exist in UserDimTbl
         if( result != null && result.next() ) {
            uID = result.getInt ( 1 );
            flag = true;
            count = 0;
         }
         result.close();

         // if selection to get userID,
         // if the user information does not exist in UserDimTbl
         if( !flag ) {
            // finding userID for the new user
	    String uIDQry = "SELECT max ( userID ) FROM UserDimTbl";
	    result = stmt.executeQuery( uIDQry );
            if ( result != null && result.next() ) {
               uID = result.getInt ( 1 );
            }
	    result.close();
            uID++;

            // inserting user information into UserDimTbl
            String update = "INSERT INTO UserDimTbl VALUES ( " + uID + ", \'"
                            + ip + "\', \'" + hostName + "\', \'"
                            + userName + "\' )";
	    count = stmt.executeUpdate( update );
	 }
	 stmt.close();

      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }
      if( count != -1 ) {
         return uID;
      } else {
         return -1;
      }
   }

   /**
     * Method insertToProtocolDimTbl will return protocolID such that
     *    1) if the protocol information exist in the ProtocolDimTbl,
     *       it will return the existing protocolID
     *    2) if the protocol information does not exist in the ProtocolDimTbl,
     *       it will insert the information to the ProtocolDimTbl and
     *       return the new protocolID
     *
     * @param       prot            prot is HTTP protocol
     *
     * @param       mt              mt is method
     *
     * @param       stStr           stStr is status
     *
     * @exception   SQLException    for sql exception
     *
     * @return                      protocolID for the protocol
     */

   public int insertToProtocolDimTbl( String prot, String mt, String stStr ) {
      int protID = -1;
      int count = -1;
      boolean flag = false;
      Statement stmt = null;
      ResultSet result;

      String protocolName;
      String methodName;
      int status;
      int st = -1;

      if( stStr != null ) {
         st = Integer.parseInt( stStr );
      }
      try {
         stmt = con.createStatement();

         // query string to check if the protocol information exist
         // in the ProtocolDimTbl
         String qry = "SELECT protocolID FROM ProtocolDimTbl WHERE "
                      + "protocolName = \'" + prot + "\' AND "
                      + "methodName = \'" + mt + "\' AND " + "status = " + st;
         result = stmt.executeQuery( qry );
         // if selection to get protocolID,
         // if the protocol information exist in ProtocolDimTbl
         if( result != null && result.next() ) {
            protID = result.getInt ( 1 );
            flag = true;
            count = 0;
         }
         result.close();

         // if selection to get protocolID,
         // if the protocol information does not exist in ProtocolDimTbl
         if( !flag ) {
           // finding protocolID for the protocol for the new protocol
	    String protIDQry = "SELECT max ( protocolID ) FROM ProtocolDimTbl";
	    result = stmt.executeQuery( protIDQry );
            int i = protID;
            if ( result != null && result.next() ) {
               protID = result.getInt ( 1 );
            }
            result.close();
	    protID++;

            // inserting protocol information into ProtocolDimTbl
            String update = "INSERT INTO ProtocolDimTbl VALUES ( " + protID
                          + ", \'" + prot + "\', \'" + mt + "\', " + st + " )";
	    count = stmt.executeUpdate( update );
	 }
	 stmt.close();
      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }
      if( count != -1 ) {
         return protID;
      } else {
         return -1;
      }
   }

   /**
     * Method insertToPathDimTbl will return pathID such that
     *    1) if the path information exist in the PathDimTbl,
     *       it will return the existing pathID
     *    2) if the path information does not exist in the PathDimTbl,
     *       it will insert the information to the PathDimTbl and
     *       return the new pathID
     *
     * @param       p             p stores path name
     *
     * @param       f             f stores file name
     *
     * @param       t             t stores file type name
     *
     * @exception   SQLException    for sql exception
     *
     * @return                      pathID for the path
     */

   public int insertToPathDimTbl( String p, String f, String t ) {
      int paID = -1;
      int count = -1;
      boolean flag = false;
      Statement stmt = null;
      ResultSet result = null;

      try {
         stmt = con.createStatement();

         // query string to check if the path information exist
         // in the PathDimTbl
         String qry = "SELECT pathID FROM PathDimTbl WHERE pathName = \'" + p
                    + "\' AND " + "fileName = \'" + f
                    + "\' AND " + "fileType = \'" + t + "\'";
         result = stmt.executeQuery( qry );
         // if selection to get pathID,
         // if the path information exist in PathDimTbl
         if( result != null && result.next() ) {
            paID = result.getInt ( 1 );
            flag = true;
            count = 0;
         }
         result.close();

         // if selection to get pathID,
         // if the path information does not exist in PathDimTbl
         if( !flag ) {
            // finding pathID for the new path
	    String paIDQry = "SELECT max ( pathID ) FROM PathDimTbl";
	    result = stmt.executeQuery( paIDQry );
            if ( result != null && result.next() ) {
               paID = result.getInt ( 1 );
            }
	    result.close();
            paID++;

            // inserting path information into PathDimTbl
            String update = "INSERT INTO PathDimTbl VALUES ( " + paID
                            + ", \'" + p + "\', \'" +  f + "\', \'" + t + "\' )";
	    count = stmt.executeUpdate( update );
	 }
	 stmt.close();
      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }
      if( count != -1 ) {
         return paID;
      } else {
         return -1;
      }
   }

   /**
     * Method insertToLogFactTbl will insert an access record into
     * LogFactTbl along with sessionID.
     *
     * @param       timeID          logical ID for time stamp
     *
     * @param       userID          foreign key refers to corresponding
     *                              primary key in UserDimtbl
     *
     * @param       protocolID      foreign key refers to corresponding
     *                              primary key in ProtocolDimtbl
     *
     * @param       pathID          foreign key refers to corresponding
     *                              primary key in PathDimtbl
     *
     * @param       b               number of bytes accessed
     *
     * @exception   SQLException    for sql exception
     *
     * @return                      int to indicate success of insertion
     */

   public int insertToLogFactTbl( String timeID, int userID,
                                  int protocolID, int pathID, int b ) {
      int count = -1;
      int sessionID = -1;
      Statement stmt = null;

      // method call to get sessionID
      sessionID = getSessionID( timeID, userID );

      try {
         stmt = con.createStatement();
         // inserting the new record into LogFactTbl
         String update = "INSERT INTO LogFactTbl VALUES ( \'" + timeID
                         + "\', " + userID + ", " + protocolID
                         + ", " + pathID + ", " + sessionID + ", " + b + " )";

         count = stmt.executeUpdate( update );
         stmt.close();
      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }
      return count;
   }

   /**
     * Method getSessionID will find sessionID for an access by a user
     *
     * @param       timeID          time stamp of an access record
     *
     * @param       userID          userID to identify user
     *
     * @exception   SQLException    for sql exception
     *
     * @return                      sessionID as int
     */

   public int getSessionID( String timeID, int userID ) {
      Statement stmt = null;
      ResultSet result = null;

      Statement s = null;
      ResultSet r = null;
      String tID = "";
      int sID = -1;
      int maxSID = -1;

      // time stamp of the current access
      MyDate d = new MyDate( timeID );
      MyDate d1 = null;
      MyDate d2 = null;

      try {
         // getting next sessionID to be used
         s = con.createStatement();
         String temp = "SELECT max ( sessionID ) FROM LogFactTbl";
         r = s.executeQuery( temp );
         if( r != null && r.next() ) {
            maxSID = r.getInt( 1 );
         }
         r.close();
         s.close();
         maxSID++;

         // getting most recent time stamp and corresponding sessionID for the user
         stmt = con.createStatement();
         String qry = "SELECT timeID, sessionID FROM LogFactTbl WHERE userID = "
                      + userID;
         result = stmt.executeQuery( qry );
         while ( result != null && result.next() ) {
            tID = result.getString ( 1 );
            d2 = new MyDate( tID );
            // compare to get most recent time stamp and sessionID
            if( d1 == null || d2.compareTo( d1 ) >= 0 ) {
               d1 = d2;
               sID = result.getInt ( 2 );
            }
         }
	 result.close();
         stmt.close();
      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }

      // if most recent access of the user is not in the same session
      // with the current access, sessionID will be next sessionID to be used
      if( d == null || d1 == null || d.notInSameSession( d1 ) ) {
         sID = maxSID;
      }
      return  sID;
   }
} // Model
